<?php
// $Id: comment_country.admin.inc,v 1.1.2.2 2010/11/29 20:05:08 worldfallz Exp $
/**
 * @file
 * Administrative page callbacks for the Comment Country module.
 */

/**
 * Comment_country administration settings.
 *
 * @return
 *   Form for comment_country configuration options.
 */
function comment_country_settings() {
  $form['comment_country_ipinfodb_key'] = array(
    '#type' => 'fieldset',
    '#title' => t('IPinfoDB.com API Key'),
  );
  $form['comment_country_ipinfodb_key']['comment_country_ipinfodb_apikey'] = array(
    '#type' => 'textfield',
    '#size' => 100,
    '#description' => t('Please provide a valid IPInfoDB.com API key. If you don\'t have one, you can register at <a href="@ipinfo">IPInfoDB.com</a>.', array('@ipinfo' => url('http://www.ipinfodb.com/register.php'))),
    '#default_value' => variable_get('comment_country_ipinfodb_apikey', ''),
  );
  $form['comment_country_admin'] = array(
    '#type' => 'fieldset',
    '#title' => t('Select what country info to display in comments:'),
  );
  $form['comment_country_admin']['comment_country_name'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display country name in comments.'),
    '#return_value' => '1',
    '#default_value' => variable_get('comment_country_name', 1),
  );
  $form['comment_country_admin']['comment_country_code'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display country code in comments.'),
    '#return_value' => '1',
    '#default_value' => variable_get('comment_country_code', 0),
  );
  $form['comment_country_admin']['comment_country_flag'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display country flag in comments.'),
    '#return_value' => '1',
    '#default_value' => variable_get('comment_country_flag', 0),
    '#disabled' => !module_exists('countryicons'),
    '#description' => module_exists('countryicons') ? t('The Country Icons module is installed and enabled.') : '<em><span style="color: #A50000;">'. t('You must install the ') . l('Country Icons', 'http://drupal.org/project/countryicons') . t(' module to use this feature.') . '</span></em>'
  );
  $form['comment_country_update'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update comment country info:'),
    '#description' => '<span style="color: red; font-weight: bold;">' . t('WARNING: this will delete all current comment country data. Use with care.') . "</span>",
  );
  $form['comment_country_update']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update Comment Country Info'),
  );
  $form['#submit'][] = 'comment_country_admin_submit';
  return system_settings_form($form);
}

/**
 * admin settings form handler
 *
 * Use the additional button to update all comment country info.
 */
function comment_country_admin_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#value'] == 'Update Comment Country Info') {
    $batch = array(
      'title' => t('Updating comment country information.'),
      'operations' => array(
        array('comment_country_batch', array()),
        ),
      'finished' => 'comment_country_batch_finished',
      'progress_message' => t('@remaining of @total updates remaining.'),
      'error_message' => t('Comment Country batch update has encountered an error.'),
      'file' => drupal_get_path('module', 'comment_country') . '/comment_country.admin.inc',
    );
    batch_set($batch);
  }
}

function comment_country_batch(&$context) {
  if (!isset($context['sandbox']['progress'])) {
    db_query('TRUNCATE TABLE {comment_country}');
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_comment'] = 0;
    $context['sandbox']['max'] = db_result(db_query('SELECT COUNT(cid) FROM {comments}'));
  }
  //process comments in groups of 10
  $limit = 10;
  $result = db_query_range("SELECT {cid, hostname} FROM {comments} WHERE cid > %d AND status = 0 ORDER BY cid ASC", $context['sandbox']['current_comment'], 0, $limit);

  //process comments
  $ipinfodb_api_key = variable_get('comment_country_ipinfodb_apikey', '');
  while ($row = db_fetch_object($result)) {
    $xml =  simplexml_load_string(drupal_http_request('http://api.ipinfodb.com/v2/ip_query_country.php?key=' . $ipinfodb_api_key . '&ip=' . $row->hostname)->data);
    db_query("INSERT INTO {comment_country} (cid, country_code, country_name) VALUES (%d, '%s', '%s')", $row->cid, $xml->CountryCode, $xml->CountryName);

    // update progress
    $context['sandbox']['progress']++;
    $context['sandbox']['current_comment'] = $row->cid;
  }

  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

function comment_country_batch_finished($success, $results, $operations) {
  if ($success) {
    $message = 'Successfully updated country information for all existing published comments.';
  }
  else {
    $error_operation = reset($operations);
    $message = 'An error occurred while processing ' . $error_operation[0] . ' with arguments :' . print_r($error_operation[0], TRUE);
  }
  drupal_set_message(check_plain($message));
}
